@@ -25,25 +25,24 @@ from traceback import format_exception
 from typing import TYPE_CHECKING, Any
 
 from sqlalchemy import Column, Integer, String, Text, delete, func, or_, select, update
-from sqlalchemy.orm import relationship, selectinload
+from sqlalchemy.orm import Session, relationship, selectinload
 from sqlalchemy.sql.functions import coalesce
 
+from airflow._shared.timezones import timezone
 from airflow.assets.manager import AssetManager
 from airflow.models.asset import asset_trigger_association_table
 from airflow.models.base import Base
 from airflow.models.taskinstance import TaskInstance
-from airflow.triggers import base as events
-from airflow.utils import timezone
+from airflow.triggers.base import BaseTaskEndEvent
 from airflow.utils.retries import run_with_db_retries
 from airflow.utils.session import NEW_SESSION, provide_session
 from airflow.utils.sqlalchemy import UtcDateTime, with_row_locks
 from airflow.utils.state import TaskInstanceState
 
 if TYPE_CHECKING:
-    from sqlalchemy.orm import Session
     from sqlalchemy.sql import Select
 
-    from airflow.triggers.base import BaseTrigger
+    from airflow.triggers.base import BaseTrigger, TriggerEvent
 
 TRIGGER_FAIL_REPR = "__fail__"
 """String value to represent trigger failure.
@@ -107,6 +106,8 @@ class Trigger(Base):
 
     assets = relationship("AssetModel", secondary=asset_trigger_association_table, back_populates="triggers")
 
+    deadline = relationship("Deadline", back_populates="trigger", uselist=False)
+
     def __init__(
         self,
         classpath: str,
@@ -188,10 +189,15 @@ class Trigger(Base):
 
     @classmethod
     @provide_session
-    def fetch_trigger_ids_with_asset(cls, session: Session = NEW_SESSION) -> set[str]:
-        """Fetch all the trigger IDs associated with at least one asset."""
-        query = select(asset_trigger_association_table.columns.trigger_id)
-        return {trigger_id for trigger_id in session.scalars(query)}
+    def fetch_trigger_ids_with_non_task_associations(cls, session: Session = NEW_SESSION) -> set[str]:
+        """Fetch all trigger IDs actively associated with non-task entities like assets and deadlines."""
+        from airflow.models import Deadline
+
+        query = select(asset_trigger_association_table.columns.trigger_id).union_all(
+            select(Deadline.trigger_id).where(Deadline.trigger_id.is_not(None))
+        )
+
+        return set(session.scalars(query))
 
     @classmethod
     @provide_session
@@ -213,10 +219,10 @@ class Trigger(Base):
                     .values(trigger_id=None)
                 )
 
-        # Get all triggers that have no task instances and assets depending on them and delete them
+        # Get all triggers that have no task instances, assets, or deadlines depending on them and delete them
         ids = (
             select(cls.id)
-            .where(~cls.assets.any())
+            .where(~cls.assets.any(), ~cls.deadline.has())
             .join(TaskInstance, cls.id == TaskInstance.trigger_id, isouter=True)
             .group_by(cls.id)
             .having(func.count(TaskInstance.trigger_id) == 0)
@@ -230,7 +236,7 @@ class Trigger(Base):
 
     @classmethod
     @provide_session
-    def submit_event(cls, trigger_id, event: events.TriggerEvent, session: Session = NEW_SESSION) -> None:
+    def submit_event(cls, trigger_id, event: TriggerEvent, session: Session = NEW_SESSION) -> None:
         """
         Fire an event.
 
@@ -256,6 +262,8 @@ class Trigger(Base):
                 extra={"from_trigger": True, "payload": event.payload},
                 session=session,
             )
+        if trigger.deadline:
+            trigger.deadline.handle_callback_event(event, session)
 
     @classmethod
     @provide_session
@@ -353,32 +361,37 @@ class Trigger(Base):
         :param alive_triggerer_ids: The alive triggerer ids as a list or a select query.
         :param session: The database session.
         """
-        query = with_row_locks(
+        result: list[int] = []
+
+        # Add triggers associated to deadlines first, then tasks, then assets
+        # It prioritizes deadline triggers, then DAGs over event driven scheduling which is fair
+        queries = [
+            # Deadline triggers
+            select(cls.id).where(cls.deadline.has()).order_by(cls.created_date),
+            # Task Instance triggers
             select(cls.id)
             .prefix_with("STRAIGHT_JOIN", dialect="mysql")
             .join(TaskInstance, cls.id == TaskInstance.trigger_id, isouter=False)
             .where(or_(cls.triggerer_id.is_(None), cls.triggerer_id.not_in(alive_triggerer_ids)))
-            .order_by(coalesce(TaskInstance.priority_weight, 0).desc(), cls.created_date)
-            .limit(capacity),
-            session,
-            skip_locked=True,
-        )
-        ti_triggers = session.execute(query).all()
+            .order_by(coalesce(TaskInstance.priority_weight, 0).desc(), cls.created_date),
+            # Asset triggers
+            select(cls.id).where(cls.assets.any()).order_by(cls.created_date),
+        ]
 
-        query = with_row_locks(
-            select(cls.id).where(cls.assets.any()).order_by(cls.created_date).limit(capacity),
-            session,
-            skip_locked=True,
-        )
-        asset_triggers = session.execute(query).all()
+        # Process each query while avoiding unnecessary queries when capacity is reached
+        for query in queries:
+            remaining_capacity = capacity - len(result)
+            if remaining_capacity <= 0:
+                break
 
-        # Add triggers associated to assets after triggers associated to tasks
-        # It prioritizes DAGs over event driven scheduling which is fair
-        return ti_triggers + asset_triggers
+            locked_query = with_row_locks(query.limit(remaining_capacity), session, skip_locked=True)
+            result.extend(session.execute(locked_query).all())
+
+        return result
 
 
 @singledispatch
-def handle_event_submit(event: events.TriggerEvent, *, task_instance: TaskInstance, session: Session) -> None:
+def handle_event_submit(event: TriggerEvent, *, task_instance: TaskInstance, session: Session) -> None:
     """
     Handle the submit event for a given task instance.
 
@@ -409,10 +422,8 @@ def handle_event_submit(event: events.TriggerEvent, *, task_instance: TaskInstan
     session.flush()
 
 
-@handle_event_submit.register(events.BaseTaskEndEvent)
-def _process_BaseTaskEndEvent(
-    event: events.BaseTaskEndEvent, *, task_instance: TaskInstance, session: Session
-) -> None:
+@handle_event_submit.register
+def _(event: BaseTaskEndEvent, *, task_instance: TaskInstance, session: Session) -> None:
     """
     Submit event for the given task instance.
 
