@@ -494,8 +494,7 @@ class TestBulkPools(TestPoolsEndpoint):
     @pytest.mark.parametrize(
         "actions, expected_results",
         [
-            # Test successful create
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -509,9 +508,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     ]
                 },
                 {"create": {"success": ["pool3", "pool4"], "errors": []}},
+                id="test_successful_create",
             ),
-            # Test successful create with skip
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -525,9 +524,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     ]
                 },
                 {"create": {"success": ["pool3"], "errors": []}},
+                id="test_successful_create_with_skip",
             ),
-            # Test successful create with overwrite
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -541,9 +540,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     ]
                 },
                 {"create": {"success": ["pool3", "pool2"], "errors": []}},
+                id="test_successful_create_with_overwrite",
             ),
-            # Test create conflict
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -564,9 +563,9 @@ class TestBulkPools(TestPoolsEndpoint):
                         ],
                     }
                 },
+                id="test_create_conflict",
             ),
-            # Test successful update
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -577,9 +576,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     ]
                 },
                 {"update": {"success": ["pool2"], "errors": []}},
+                id="test_successful_update",
             ),
-            # Test update with skip
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -590,9 +589,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     ]
                 },
                 {"update": {"success": [], "errors": []}},
+                id="test_update_with_skip",
             ),
-            # Test update not found
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -613,19 +612,19 @@ class TestBulkPools(TestPoolsEndpoint):
                         ],
                     }
                 },
+                id="test_update_not_found",
             ),
-            # Test successful delete
-            (
+            pytest.param(
                 {"actions": [{"action": "delete", "entities": ["pool1"], "action_on_non_existence": "skip"}]},
                 {"delete": {"success": ["pool1"], "errors": []}},
+                id="test_successful_delete",
             ),
-            # Test delete with skip
-            (
+            pytest.param(
                 {"actions": [{"action": "delete", "entities": ["pool3"], "action_on_non_existence": "skip"}]},
                 {"delete": {"success": [], "errors": []}},
+                id="test_delete_with_skip",
             ),
-            # Test delete not found
-            (
+            pytest.param(
                 {"actions": [{"action": "delete", "entities": ["pool4"], "action_on_non_existence": "fail"}]},
                 {
                     "delete": {
@@ -638,9 +637,9 @@ class TestBulkPools(TestPoolsEndpoint):
                         ],
                     }
                 },
+                id="test_delete_not_found",
             ),
-            # Test Create, Update, and Delete combined
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -661,9 +660,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     "update": {"success": ["pool1"], "errors": []},
                     "delete": {"success": ["pool2"], "errors": []},
                 },
+                id="test_create_update_delete",
             ),
-            # Test Fail on conflicting create and handle others
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -692,9 +691,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     "update": {"success": ["pool1"], "errors": []},
                     "delete": {"success": [], "errors": []},
                 },
+                id="test_create_update_delete_with_fail",
             ),
-            # Test all skipping actions
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -715,9 +714,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     "update": {"success": [], "errors": []},
                     "delete": {"success": [], "errors": []},
                 },
+                id="test_create_update_delete_with_skip",
             ),
-            # Test Dependent actions
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -740,9 +739,9 @@ class TestBulkPools(TestPoolsEndpoint):
                     "update": {"success": ["pool5"], "errors": []},
                     "delete": {"success": ["pool5"], "errors": []},
                 },
+                id="test_dependent_actions",
             ),
-            # Test Repeated actions
-            (
+            pytest.param(
                 {
                     "actions": [
                         {
@@ -815,6 +814,7 @@ class TestBulkPools(TestPoolsEndpoint):
                     },
                     "delete": {"success": ["pool2"], "errors": []},
                 },
+                id="test_repeated_actions",
             ),
         ],
     )
@@ -831,5 +831,17 @@ class TestBulkPools(TestPoolsEndpoint):
         assert response.status_code == 401
 
     def test_should_respond_403(self, unauthorized_test_client):
-        response = unauthorized_test_client.patch("/pools", json={})
+        response = unauthorized_test_client.patch(
+            "/pools",
+            json={
+                "actions": [
+                    {
+                        "action": "create",
+                        "entities": [
+                            {"pool": "pool1", "slots": 1},
+                        ],
+                    },
+                ]
+            },
+        )
         assert response.status_code == 403
