@@ -38,10 +38,12 @@ from pydantic import TypeAdapter
 from pydantic.v1.utils import deep_update
 from requests.adapters import Response
 
+from airflow import settings
 from airflow.config_templates.airflow_local_settings import DEFAULT_LOGGING_CONFIG
 from airflow.executors import executor_constants, executor_loader
 from airflow.jobs.job import Job
 from airflow.jobs.triggerer_job_runner import TriggererJobRunner
+from airflow.models.dag_version import DagVersion
 from airflow.models.dagrun import DagRun
 from airflow.models.taskinstance import TaskInstance
 from airflow.models.taskinstancehistory import TaskInstanceHistory
@@ -77,7 +79,7 @@ from tests_common.test_utils.file_task_handler import (
 )
 from tests_common.test_utils.markers import skip_if_force_lowest_dependencies_marker
 
-pytestmark = pytest.mark.db_test
+pytestmark = [pytest.mark.db_test]
 
 DEFAULT_DATE = pendulum.datetime(2016, 1, 1)
 TASK_LOGGER = "airflow.task"
@@ -91,8 +93,7 @@ class TestFileTaskLogHandler:
             session.query(TaskInstance).delete()
 
     def setup_method(self):
-        logging.config.dictConfig(DEFAULT_LOGGING_CONFIG)
-        logging.root.disabled = False
+        settings.configure_logging()
         self.clean_up()
         # We use file task handler by default.
 
@@ -119,7 +120,8 @@ class TestFileTaskLogHandler:
             )
 
         dagrun = dag_maker.create_dagrun()
-        ti = TaskInstance(task=task, run_id=dagrun.run_id)
+        dag_version = DagVersion.get_latest_version(dagrun.dag_id)
+        ti = TaskInstance(task=task, run_id=dagrun.run_id, dag_version_id=dag_version.id)
 
         logger = ti.log
         ti.log.disabled = False
@@ -154,7 +156,8 @@ class TestFileTaskLogHandler:
         # Remove the generated tmp log file.
         os.remove(log_filename)
 
-    def test_file_task_handler_when_ti_is_skipped(self, dag_maker):
+    @pytest.mark.parametrize("ti_state", [TaskInstanceState.SKIPPED, TaskInstanceState.UPSTREAM_FAILED])
+    def test_file_task_handler_when_ti_is_not_run(self, dag_maker, ti_state):
         def task_callable(ti):
             ti.log.info("test")
 
@@ -164,13 +167,14 @@ class TestFileTaskLogHandler:
                 python_callable=task_callable,
             )
         dagrun = dag_maker.create_dagrun()
-        ti = TaskInstance(task=task, run_id=dagrun.run_id)
+        dag_version = DagVersion.get_latest_version(dagrun.dag_id)
+        ti = TaskInstance(task=task, run_id=dagrun.run_id, dag_version_id=dag_version.id)
 
         ti.try_number = 0
-        ti.state = State.SKIPPED
+        ti.state = ti_state
 
-        logger = ti.log
-        ti.log.disabled = False
+        logger = logging.getLogger(TASK_LOGGER)
+        logger.disabled = False
 
         file_handler = next(
             (handler for handler in logger.handlers if handler.name == FILE_TASK_HANDLER), None
@@ -292,8 +296,8 @@ class TestFileTaskLogHandler:
                 ti.executor = executor_name
             ti.try_number = 1
             ti.state = TaskInstanceState.RUNNING
-            logger = ti.log
-            ti.log.disabled = False
+            logger = logging.getLogger(TASK_LOGGER)
+            logger.disabled = False
 
             file_handler = next(
                 (handler for handler in logger.handlers if handler.name == FILE_TASK_HANDLER), None
@@ -335,13 +339,14 @@ class TestFileTaskLogHandler:
                 python_callable=task_callable,
             )
         dagrun = dag_maker.create_dagrun()
-        ti = TaskInstance(task=task, run_id=dagrun.run_id)
+        dag_version = DagVersion.get_latest_version(dagrun.dag_id)
+        ti = TaskInstance(task=task, run_id=dagrun.run_id, dag_version_id=dag_version.id)
 
         ti.try_number = 2
         ti.state = State.RUNNING
 
-        logger = ti.log
-        ti.log.disabled = False
+        logger = logging.getLogger(TASK_LOGGER)
+        logger.disabled = False
 
         file_handler = next(
             (handler for handler in logger.handlers if handler.name == FILE_TASK_HANDLER), None
@@ -386,13 +391,14 @@ class TestFileTaskLogHandler:
                 python_callable=task_callable,
             )
         dagrun = dag_maker.create_dagrun()
-        ti = TaskInstance(task=task, run_id=dagrun.run_id)
+        dag_version = DagVersion.get_latest_version(dagrun.dag_id)
+        ti = TaskInstance(task=task, run_id=dagrun.run_id, dag_version_id=dag_version.id)
 
         ti.try_number = 1
         ti.state = State.RUNNING
 
-        logger = ti.log
-        ti.log.disabled = False
+        logger = logging.getLogger(TASK_LOGGER)
+        logger.disabled = False
 
         file_handler = next(
             (handler for handler in logger.handlers if handler.name == FILE_TASK_HANDLER), None
@@ -408,7 +414,7 @@ class TestFileTaskLogHandler:
         assert log_filename.endswith("1.log"), log_filename
 
         # mock to generate 2000 lines of log, the total size is larger than max_bytes_size
-        for i in range(1, 2000):
+        for i in range(1, 3000):
             logger.info("this is a Test. %s", i)
 
         # this is the rotate log file
