@@ -36,11 +36,11 @@ from uuid6 import uuid7
 from airflowctl import __version__ as version
 from airflowctl.api.operations import (
     AssetsOperations,
-    BackfillsOperations,
+    BackfillOperations,
     ConfigOperations,
     ConnectionsOperations,
-    DagOperations,
     DagRunOperations,
+    DagsOperations,
     JobsOperations,
     LoginOperations,
     PoolsOperations,
@@ -94,6 +94,8 @@ def get_json_error(response: httpx.Response):
     """Raise a ServerResponseError if we can extract error info from the error."""
     err = ServerResponseError.from_response(response)
     if err:
+        # This part is used in integration tests to verify the error message
+        # If you are updating here don't forget to update the airflow-ctl-tests
         log.warning("Server error ", extra=dict(err.response.json()))
         raise err
 
@@ -133,8 +135,15 @@ class Credentials:
         os.makedirs(default_config_dir, exist_ok=True)
         with open(os.path.join(default_config_dir, self.input_cli_config_file), "w") as f:
             json.dump({"api_url": self.api_url}, f)
+
         try:
-            keyring.set_password("airflowctl", f"api_token-{self.api_environment}", self.api_token)
+            if os.getenv("AIRFLOW_CLI_DEBUG_MODE") == "true":
+                with open(
+                    os.path.join(default_config_dir, f"debug_creds_{self.input_cli_config_file}"), "w"
+                ) as f:
+                    json.dump({f"api_token_{self.api_environment}": self.api_token}, f)
+            else:
+                keyring.set_password("airflowctl", f"api_token_{self.api_environment}", self.api_token)
         except NoKeyringError as e:
             log.error(e)
         except TypeError as e:
@@ -145,12 +154,20 @@ class Credentials:
     def load(self) -> Credentials:
         """Load the credentials from keyring and URL from disk file."""
         default_config_dir = os.environ.get("AIRFLOW_HOME", os.path.expanduser("~/airflow"))
-        credential_path = os.path.join(default_config_dir, self.input_cli_config_file)
+        config_path = os.path.join(default_config_dir, self.input_cli_config_file)
         try:
-            with open(credential_path) as f:
+            with open(config_path) as f:
                 credentials = json.load(f)
                 self.api_url = credentials["api_url"]
-                self.api_token = keyring.get_password("airflowctl", f"api_token-{self.api_environment}")
+                if os.getenv("AIRFLOW_CLI_DEBUG_MODE") == "true":
+                    debug_creds_path = os.path.join(
+                        default_config_dir, f"debug_creds_{self.input_cli_config_file}"
+                    )
+                    with open(debug_creds_path) as df:
+                        debug_credentials = json.load(df)
+                        self.api_token = debug_credentials.get(f"api_token_{self.api_environment}")
+                else:
+                    self.api_token = keyring.get_password("airflowctl", f"api_token_{self.api_environment}")
         except FileNotFoundError:
             if self.client_kind == ClientKind.AUTH:
                 # Saving the URL set from the Auth Commands if Kind is AUTH
@@ -227,7 +244,7 @@ class Client(httpx.Client):
     @property
     def backfills(self):
         """Operations related to backfills."""
-        return BackfillsOperations(self)
+        return BackfillOperations(self)
 
     @lru_cache()  # type: ignore[prop-decorator]
     @property
@@ -245,7 +262,7 @@ class Client(httpx.Client):
     @property
     def dags(self):
         """Operations related to DAGs."""
-        return DagOperations(self)
+        return DagsOperations(self)
 
     @lru_cache()  # type: ignore[prop-decorator]
     @property
@@ -299,7 +316,7 @@ def get_client(kind: Literal[ClientKind.CLI, ClientKind.AUTH] = ClientKind.CLI):
         api_client = Client(
             base_url=credentials.api_url or "http://localhost:8080",
             limits=httpx.Limits(max_keepalive_connections=1, max_connections=1),
-            token=credentials.api_token or "",
+            token=credentials.api_token or str(os.getenv("AIRFLOW_CLI_TOKEN", "")),
             kind=kind,
         )
         yield api_client
