@@ -18,8 +18,8 @@
 from __future__ import annotations
 
 import itertools
-from collections.abc import Iterable
-from typing import Callable, TypeVar
+from collections.abc import Callable, Iterable
+from typing import TypeVar
 
 T = TypeVar("T")
 
