@@ -250,7 +250,7 @@ class TestCloudSpanner:
 
     @mock.patch("airflow.providers.google.cloud.operators.spanner.SpannerHook")
     def test_instance_query(self, mock_hook):
-        mock_hook.return_value.execute_sql.return_value = None
+        mock_hook.return_value.execute_dml.return_value = [3]
         op = SpannerQueryDatabaseInstanceOperator(
             project_id=PROJECT_ID,
             instance_id=INSTANCE_ID,
@@ -258,8 +258,7 @@ class TestCloudSpanner:
             query=INSERT_QUERY,
             task_id="id",
         )
-        context = mock.MagicMock()
-        result = op.execute(context=context)
+        result = op.execute(context=mock.MagicMock())
         mock_hook.assert_called_once_with(
             gcp_conn_id="google_cloud_default",
             impersonation_chain=None,
@@ -267,11 +266,11 @@ class TestCloudSpanner:
         mock_hook.return_value.execute_dml.assert_called_once_with(
             project_id=PROJECT_ID, instance_id=INSTANCE_ID, database_id=DB_ID, queries=[INSERT_QUERY]
         )
-        assert result is None
+        assert result == [3]
 
     @mock.patch("airflow.providers.google.cloud.operators.spanner.SpannerHook")
     def test_instance_query_missing_project_id(self, mock_hook):
-        mock_hook.return_value.execute_sql.return_value = None
+        mock_hook.return_value.execute_dml.return_value = [3]
         op = SpannerQueryDatabaseInstanceOperator(
             instance_id=INSTANCE_ID, database_id=DB_ID, query=INSERT_QUERY, task_id="id"
         )
@@ -284,7 +283,7 @@ class TestCloudSpanner:
         mock_hook.return_value.execute_dml.assert_called_once_with(
             project_id=None, instance_id=INSTANCE_ID, database_id=DB_ID, queries=[INSERT_QUERY]
         )
-        assert result is None
+        assert result == [3]
 
     @pytest.mark.parametrize(
         "project_id, instance_id, database_id, query, exp_msg",
