@@ -20,7 +20,7 @@ import datetime
 import json
 import uuid
 from json import JSONEncoder
-from typing import TYPE_CHECKING, Any
+from typing import Any
 from unittest.mock import MagicMock, patch
 
 import pytest
@@ -28,7 +28,6 @@ from attrs import define
 from openlineage.client.utils import RedactMixin
 from pkg_resources import parse_version
 
-from airflow.models import DAG, DagModel
 from airflow.providers.common.compat.assets import Asset
 from airflow.providers.openlineage.plugins.facets import AirflowDebugRunFacet
 from airflow.providers.openlineage.utils.utils import (
@@ -44,25 +43,38 @@ from airflow.providers.openlineage.utils.utils import (
     is_operator_disabled,
 )
 from airflow.serialization.enums import DagAttributeTypes, Encoding
-from airflow.utils import timezone
 from airflow.utils.state import State
 from airflow.utils.types import DagRunType
 
 from tests_common.test_utils.compat import (
     BashOperator,
 )
-from tests_common.test_utils.version_compat import AIRFLOW_V_2_10_PLUS, AIRFLOW_V_3_0_PLUS
+from tests_common.test_utils.version_compat import AIRFLOW_V_3_0_PLUS, AIRFLOW_V_3_1_PLUS
+
+if AIRFLOW_V_3_1_PLUS:
+    from airflow.models.dag import get_next_data_interval
+    from airflow.sdk import timezone
+else:
+    from airflow.utils import timezone  # type: ignore[attr-defined,no-redef]
+
+if AIRFLOW_V_3_1_PLUS:
+    from airflow.sdk._shared.secrets_masker import DEFAULT_SENSITIVE_FIELDS, SecretsMasker
+elif AIRFLOW_V_3_0_PLUS:
+    from airflow.sdk.execution_time.secrets_masker import (  # type: ignore[no-redef]
+        DEFAULT_SENSITIVE_FIELDS,
+        SecretsMasker,
+    )
+else:
+    from airflow.utils.log.secrets_masker import (  # type: ignore[attr-defined,no-redef]
+        DEFAULT_SENSITIVE_FIELDS,
+        SecretsMasker,
+    )
 
 if AIRFLOW_V_3_0_PLUS:
+    from airflow.sdk import DAG
     from airflow.utils.types import DagRunTriggeredByType
-
-if TYPE_CHECKING:
-    from airflow.sdk.execution_time.secrets_masker import _secrets_masker
 else:
-    try:
-        from airflow.sdk.execution_time.secrets_masker import _secrets_masker
-    except ImportError:
-        from airflow.utils.log.secrets_masker import _secrets_masker
+    from airflow import DAG
 
 
 class SafeStrDict(dict):
@@ -99,16 +111,19 @@ def test_get_airflow_debug_facet_logging_set_to_debug(mock_debug_mode, mock_get_
 
 
 @pytest.mark.db_test
+@pytest.mark.need_serialized_dag
 def test_get_dagrun_start_end(dag_maker):
     start_date = datetime.datetime(2022, 1, 1)
     end_date = datetime.datetime(2022, 1, 1, hour=2)
     with dag_maker("test", start_date=start_date, end_date=end_date, schedule="@once") as dag:
         pass
     dag_maker.sync_dagbag_to_db()
-    dag_model = DagModel.get_dagmodel(dag.dag_id)
 
     run_id = str(uuid.uuid1())
-    data_interval = dag.get_next_data_interval(dag_model)
+    if AIRFLOW_V_3_1_PLUS:
+        data_interval = get_next_data_interval(dag.timetable, dag_maker.dag_model)
+    else:
+        data_interval = dag.get_next_data_interval(dag_maker.dag_model)
     if AIRFLOW_V_3_0_PLUS:
         dagrun_kwargs = {
             "logical_date": data_interval.start,
@@ -243,7 +258,10 @@ def test_is_name_redactable():
 
 @pytest.mark.enable_redact
 def test_redact_with_exclusions(monkeypatch):
-    redactor = OpenLineageRedactor.from_masker(_secrets_masker())  # type: ignore[assignment]
+    sm = SecretsMasker()
+    if AIRFLOW_V_3_1_PLUS:
+        sm.sensitive_variables_fields = list(DEFAULT_SENSITIVE_FIELDS)
+    redactor = OpenLineageRedactor.from_masker(sm)
 
     class NotMixin:
         def __init__(self):
@@ -562,7 +580,7 @@ def test_serialize_timetable_with_dataset_or_time_schedule():
 
 
 @pytest.mark.skipif(
-    not AIRFLOW_V_2_10_PLUS or AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_0_PLUS,
     reason="This test checks serialization only in 2.10 conditions",
 )
 def test_serialize_timetable_2_10_complex_with_alias():
@@ -600,7 +618,7 @@ def test_serialize_timetable_2_10_complex_with_alias():
 
 
 @pytest.mark.skipif(
-    not AIRFLOW_V_2_10_PLUS or AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_0_PLUS,
     reason="This test checks serialization only in 2.10 conditions",
 )
 def test_serialize_timetable_2_10_single_asset():
@@ -612,7 +630,7 @@ def test_serialize_timetable_2_10_single_asset():
 
 
 @pytest.mark.skipif(
-    not AIRFLOW_V_2_10_PLUS or AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_0_PLUS,
     reason="This test checks serialization only in 2.10 conditions",
 )
 def test_serialize_timetable_2_10_list_of_assets():
@@ -630,7 +648,7 @@ def test_serialize_timetable_2_10_list_of_assets():
 
 
 @pytest.mark.skipif(
-    not AIRFLOW_V_2_10_PLUS or AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_0_PLUS,
     reason="This test checks serialization only in 2.10 conditions",
 )
 def test_serialize_timetable_2_10_with_complex_logical_condition():
@@ -665,7 +683,7 @@ def test_serialize_timetable_2_10_with_complex_logical_condition():
 
 
 @pytest.mark.skipif(
-    not AIRFLOW_V_2_10_PLUS or AIRFLOW_V_3_0_PLUS,
+    AIRFLOW_V_3_0_PLUS,
     reason="This test checks serialization only in 2.10 conditions",
 )
 def test_serialize_timetable_2_10_with_dataset_or_time_schedule():
@@ -709,102 +727,6 @@ def test_serialize_timetable_2_10_with_dataset_or_time_schedule():
     }
 
 
-@pytest.mark.skipif(AIRFLOW_V_2_10_PLUS, reason="This test checks serialization only in 2.9 conditions")
-def test_serialize_timetable_2_9_single_asset():
-    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=Asset("a"))
-    dag_info = DagInfo(dag)
-    assert dag_info.timetable == {"dataset_condition": {"__type": "dataset", "uri": "a", "extra": None}}
-
-
-@pytest.mark.skipif(AIRFLOW_V_2_10_PLUS, reason="This test checks serialization only in 2.9 conditions")
-def test_serialize_timetable_2_9_list_of_assets():
-    dag = DAG(dag_id="test", start_date=datetime.datetime(2025, 1, 1), schedule=[Asset("a"), Asset("b")])
-    dag_info = DagInfo(dag)
-    assert dag_info.timetable == {
-        "dataset_condition": {
-            "__type": "dataset_all",
-            "objects": [
-                {"__type": "dataset", "extra": None, "uri": "a"},
-                {"__type": "dataset", "extra": None, "uri": "b"},
-            ],
-        }
-    }
-
-
-@pytest.mark.skipif(AIRFLOW_V_2_10_PLUS, reason="This test checks serialization only in 2.9 conditions")
-def test_serialize_timetable_2_9_with_complex_logical_condition():
-    dag = DAG(
-        dag_id="test",
-        start_date=datetime.datetime(2025, 1, 1),
-        schedule=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
-        & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
-    )
-    dag_info = DagInfo(dag)
-    assert dag_info.timetable == {
-        "dataset_condition": {
-            "__type": "dataset_all",
-            "objects": [
-                {
-                    "__type": "dataset_any",
-                    "objects": [
-                        {"__type": "dataset", "uri": "ds1", "extra": {"some_extra": 1}},
-                        {"__type": "dataset", "uri": "ds2", "extra": None},
-                    ],
-                },
-                {
-                    "__type": "dataset_any",
-                    "objects": [
-                        {"__type": "dataset", "uri": "ds3", "extra": None},
-                        {"__type": "dataset", "uri": "ds4", "extra": {"another_extra": 345}},
-                    ],
-                },
-            ],
-        }
-    }
-
-
-@pytest.mark.skipif(AIRFLOW_V_2_10_PLUS, reason="This test checks serialization only in 2.9 conditions")
-def test_serialize_timetable_2_9_with_dataset_or_time_schedule():
-    from airflow.timetables.datasets import DatasetOrTimeSchedule
-    from airflow.timetables.trigger import CronTriggerTimetable
-
-    dag = DAG(
-        dag_id="test",
-        start_date=datetime.datetime(2025, 1, 1),
-        schedule=DatasetOrTimeSchedule(
-            timetable=CronTriggerTimetable("0 0 * 3 *", timezone="UTC"),
-            datasets=(Asset("ds1", extra={"some_extra": 1}) | Asset("ds2"))
-            & (Asset("ds3") | Asset("ds4", extra={"another_extra": 345})),
-        ),
-    )
-    dag_info = DagInfo(dag)
-    assert dag_info.timetable == {
-        "timetable": {
-            "__type": "airflow.timetables.trigger.CronTriggerTimetable",
-            "__var": {"expression": "0 0 * 3 *", "timezone": "UTC", "interval": 0.0},
-        },
-        "dataset_condition": {
-            "__type": "dataset_all",
-            "objects": [
-                {
-                    "__type": "dataset_any",
-                    "objects": [
-                        {"__type": "dataset", "uri": "ds1", "extra": {"some_extra": 1}},
-                        {"__type": "dataset", "uri": "ds2", "extra": None},
-                    ],
-                },
-                {
-                    "__type": "dataset_any",
-                    "objects": [
-                        {"__type": "dataset", "uri": "ds3", "extra": None},
-                        {"__type": "dataset", "uri": "ds4", "extra": {"another_extra": 345}},
-                    ],
-                },
-            ],
-        },
-    }
-
-
 @pytest.mark.parametrize(
     ("airflow_version", "ol_version"),
     [
