@@ -23,20 +23,23 @@ from datetime import datetime
 from typing import TYPE_CHECKING
 from unittest import mock
 
+import certifi
 import httpx
 import pytest
+import time_machine
 import uuid6
 from task_sdk import make_client, make_client_w_dry_run, make_client_w_responses
 from uuid6 import uuid7
 
 from airflow.sdk import timezone
-from airflow.sdk.api.client import RemoteValidationError, ServerResponseError
+from airflow.sdk.api.client import Client, RemoteValidationError, ServerResponseError
 from airflow.sdk.api.datamodels._generated import (
     AssetEventsResponse,
     AssetResponse,
     ConnectionResponse,
     DagRunState,
     DagRunStateResponse,
+    HITLDetailRequest,
     HITLDetailResponse,
     HITLUser,
     VariableResponse,
@@ -46,7 +49,6 @@ from airflow.sdk.exceptions import ErrorType
 from airflow.sdk.execution_time.comms import (
     DeferTask,
     ErrorResponse,
-    HITLDetailRequestResult,
     OKResponse,
     PreviousDagRunResult,
     RescheduleTask,
@@ -99,6 +101,23 @@ class TestClient:
 
         assert isinstance(err.value, FileNotFoundError)
 
+    @mock.patch("airflow.sdk.api.client.API_TIMEOUT", 60.0)
+    def test_timeout_configuration(self):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            return httpx.Response(status_code=200)
+
+        client = make_client(httpx.MockTransport(handle_request))
+        assert client.timeout == httpx.Timeout(60.0)
+
+    def test_timeout_can_be_overridden(self):
+        def handle_request(request: httpx.Request) -> httpx.Response:
+            return httpx.Response(status_code=200)
+
+        client = Client(
+            base_url="test://server", token="", transport=httpx.MockTransport(handle_request), timeout=120.0
+        )
+        assert client.timeout == httpx.Timeout(120.0)
+
     def test_error_parsing(self):
         responses = [
             httpx.Response(422, json={"detail": [{"loc": ["#0"], "msg": "err", "type": "required"}]})
@@ -154,76 +173,57 @@ class TestClient:
         assert unpickled.response.status_code == 404
         assert unpickled.request.url == "http://error"
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_retry_handling_unrecoverable_error(self, mock_sleep):
-        responses: list[httpx.Response] = [
-            *[httpx.Response(500, text="Internal Server Error")] * 6,
-            httpx.Response(200, json={"detail": "Recovered from error - but will fail before"}),
-            httpx.Response(400, json={"detail": "Should not get here"}),
-        ]
-        client = make_client_w_responses(responses)
+    def test_retry_handling_unrecoverable_error(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            responses: list[httpx.Response] = [
+                *[httpx.Response(500, text="Internal Server Error")] * 6,
+                httpx.Response(200, json={"detail": "Recovered from error - but will fail before"}),
+                httpx.Response(400, json={"detail": "Should not get here"}),
+            ]
+            client = make_client_w_responses(responses)
 
-        with pytest.raises(httpx.HTTPStatusError) as err:
-            client.get("http://error")
-        assert not isinstance(err.value, ServerResponseError)
-        assert len(responses) == 3
-        assert mock_sleep.call_count == 4
+            with pytest.raises(httpx.HTTPStatusError) as err:
+                client.get("http://error")
+            assert not isinstance(err.value, ServerResponseError)
+            assert len(responses) == 3
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_retry_handling_recovered(self, mock_sleep):
-        responses: list[httpx.Response] = [
-            *[httpx.Response(500, text="Internal Server Error")] * 2,
-            httpx.Response(200, json={"detail": "Recovered from error"}),
-            httpx.Response(400, json={"detail": "Should not get here"}),
-        ]
-        client = make_client_w_responses(responses)
+    def test_retry_handling_recovered(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            responses: list[httpx.Response] = [
+                *[httpx.Response(500, text="Internal Server Error")] * 2,
+                httpx.Response(200, json={"detail": "Recovered from error"}),
+                httpx.Response(400, json={"detail": "Should not get here"}),
+            ]
+            client = make_client_w_responses(responses)
 
-        response = client.get("http://error")
-        assert response.status_code == 200
-        assert len(responses) == 1
-        assert mock_sleep.call_count == 2
+            response = client.get("http://error")
+            assert response.status_code == 200
+            assert len(responses) == 1
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_retry_handling_overload(self, mock_sleep):
-        responses: list[httpx.Response] = [
-            httpx.Response(429, text="I am really busy atm, please back-off", headers={"Retry-After": "37"}),
-            httpx.Response(200, json={"detail": "Recovered from error"}),
-            httpx.Response(400, json={"detail": "Should not get here"}),
-        ]
-        client = make_client_w_responses(responses)
+    def test_retry_handling_non_retry_error(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            responses: list[httpx.Response] = [
+                httpx.Response(422, json={"detail": "Somehow this is a bad request"}),
+                httpx.Response(400, json={"detail": "Should not get here"}),
+            ]
+            client = make_client_w_responses(responses)
 
-        response = client.get("http://error")
-        assert response.status_code == 200
-        assert len(responses) == 1
-        assert mock_sleep.call_count == 1
-        assert mock_sleep.call_args[0][0] == 37
+            with pytest.raises(ServerResponseError) as err:
+                client.get("http://error")
+            assert len(responses) == 1
+            assert err.value.args == ("Somehow this is a bad request",)
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_retry_handling_non_retry_error(self, mock_sleep):
-        responses: list[httpx.Response] = [
-            httpx.Response(422, json={"detail": "Somehow this is a bad request"}),
-            httpx.Response(400, json={"detail": "Should not get here"}),
-        ]
-        client = make_client_w_responses(responses)
+    def test_retry_handling_ok(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            responses: list[httpx.Response] = [
+                httpx.Response(200, json={"detail": "Recovered from error"}),
+                httpx.Response(400, json={"detail": "Should not get here"}),
+            ]
+            client = make_client_w_responses(responses)
 
-        with pytest.raises(ServerResponseError) as err:
-            client.get("http://error")
-        assert len(responses) == 1
-        assert mock_sleep.call_count == 0
-        assert err.value.args == ("Somehow this is a bad request",)
-
-    @mock.patch("time.sleep", return_value=None)
-    def test_retry_handling_ok(self, mock_sleep):
-        responses: list[httpx.Response] = [
-            httpx.Response(200, json={"detail": "Recovered from error"}),
-            httpx.Response(400, json={"detail": "Should not get here"}),
-        ]
-        client = make_client_w_responses(responses)
-
-        response = client.get("http://error")
-        assert response.status_code == 200
-        assert len(responses) == 1
-        assert mock_sleep.call_count == 0
+            response = client.get("http://error")
+            assert response.status_code == 200
+            assert len(responses) == 1
 
     def test_token_renewal(self):
         responses: list[httpx.Response] = [
@@ -269,40 +269,40 @@ class TestTaskInstanceOperations:
     response parsing.
     """
 
-    @mock.patch("time.sleep", return_value=None)  # To have retries not slowing down tests
-    def test_task_instance_start(self, mock_sleep, make_ti_context):
-        # Simulate a successful response from the server that starts a task
-        ti_id = uuid6.uuid7()
-        start_date = "2024-10-31T12:00:00Z"
-        ti_context = make_ti_context(
-            start_date=start_date,
-            logical_date="2024-10-31T12:00:00Z",
-            run_type="manual",
-        )
+    def test_task_instance_start(self, make_ti_context):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            # Simulate a successful response from the server that starts a task
+            ti_id = uuid6.uuid7()
+            start_date = "2024-10-31T12:00:00Z"
+            ti_context = make_ti_context(
+                start_date=start_date,
+                logical_date="2024-10-31T12:00:00Z",
+                run_type="manual",
+            )
 
-        # ...including a validation that retry really works
-        call_count = 0
+            # ...including a validation that retry really works
+            call_count = 0
 
-        def handle_request(request: httpx.Request) -> httpx.Response:
-            nonlocal call_count
-            call_count += 1
-            if call_count < 3:
-                return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
-            if request.url.path == f"/task-instances/{ti_id}/run":
-                actual_body = json.loads(request.read())
-                assert actual_body["pid"] == 100
-                assert actual_body["start_date"] == start_date
-                assert actual_body["state"] == "running"
-                return httpx.Response(
-                    status_code=200,
-                    json=ti_context.model_dump(mode="json"),
-                )
-            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+            def handle_request(request: httpx.Request) -> httpx.Response:
+                nonlocal call_count
+                call_count += 1
+                if call_count < 3:
+                    return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
+                if request.url.path == f"/task-instances/{ti_id}/run":
+                    actual_body = json.loads(request.read())
+                    assert actual_body["pid"] == 100
+                    assert actual_body["start_date"] == start_date
+                    assert actual_body["state"] == "running"
+                    return httpx.Response(
+                        status_code=200,
+                        json=ti_context.model_dump(mode="json"),
+                    )
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
 
-        client = make_client(transport=httpx.MockTransport(handle_request))
-        resp = client.task_instances.start(ti_id, 100, start_date)
-        assert resp == ti_context
-        assert call_count == 3
+            client = make_client(transport=httpx.MockTransport(handle_request))
+            resp = client.task_instances.start(ti_id, 100, start_date)
+            assert resp == ti_context
+            assert call_count == 3
 
     @pytest.mark.parametrize(
         "state", [state for state in TerminalTIState if state != TerminalTIState.SUCCESS]
@@ -545,31 +545,31 @@ class TestVariableOperations:
     response parsing.
     """
 
-    @mock.patch("time.sleep", return_value=None)  # To have retries not slowing down tests
-    def test_variable_get_success(self, mock_sleep):
-        # Simulate a successful response from the server with a variable
-        # ...including a validation that retry really works
-        call_count = 0
+    def test_variable_get_success(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            # Simulate a successful response from the server with a variable
+            # ...including a validation that retry really works
+            call_count = 0
 
-        def handle_request(request: httpx.Request) -> httpx.Response:
-            nonlocal call_count
-            call_count += 1
-            if call_count < 2:
-                return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
-            if request.url.path == "/variables/test_key":
-                return httpx.Response(
-                    status_code=200,
-                    json={"key": "test_key", "value": "test_value"},
-                )
-            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+            def handle_request(request: httpx.Request) -> httpx.Response:
+                nonlocal call_count
+                call_count += 1
+                if call_count < 2:
+                    return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
+                if request.url.path == "/variables/test_key":
+                    return httpx.Response(
+                        status_code=200,
+                        json={"key": "test_key", "value": "test_value"},
+                    )
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
 
-        client = make_client(transport=httpx.MockTransport(handle_request))
-        result = client.variables.get(key="test_key")
+            client = make_client(transport=httpx.MockTransport(handle_request))
+            result = client.variables.get(key="test_key")
 
-        assert isinstance(result, VariableResponse)
-        assert result.key == "test_key"
-        assert result.value == "test_value"
-        assert call_count == 2
+            assert isinstance(result, VariableResponse)
+            assert result.key == "test_key"
+            assert result.value == "test_value"
+            assert call_count == 2
 
     def test_variable_not_found(self):
         # Simulate a 404 response from the server
@@ -594,26 +594,26 @@ class TestVariableOperations:
         assert resp.error == ErrorType.VARIABLE_NOT_FOUND
         assert resp.detail == {"key": "non_existent_var"}
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_variable_get_500_error(self, mock_sleep):
-        # Simulate a response from the server returning a 500 error
-        def handle_request(request: httpx.Request) -> httpx.Response:
-            if request.url.path == "/variables/test_key":
-                return httpx.Response(
-                    status_code=500,
-                    headers=[("content-Type", "application/json")],
-                    json={
-                        "reason": "internal_server_error",
-                        "message": "Internal Server Error",
-                    },
-                )
-            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+    def test_variable_get_500_error(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            # Simulate a response from the server returning a 500 error
+            def handle_request(request: httpx.Request) -> httpx.Response:
+                if request.url.path == "/variables/test_key":
+                    return httpx.Response(
+                        status_code=500,
+                        headers=[("content-Type", "application/json")],
+                        json={
+                            "reason": "internal_server_error",
+                            "message": "Internal Server Error",
+                        },
+                    )
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
 
-        client = make_client(transport=httpx.MockTransport(handle_request))
-        with pytest.raises(ServerResponseError):
-            client.variables.get(
-                key="test_key",
-            )
+            client = make_client(transport=httpx.MockTransport(handle_request))
+            with pytest.raises(ServerResponseError):
+                client.variables.get(
+                    key="test_key",
+                )
 
     def test_variable_set_success(self):
         # Simulate a successful response from the server when putting a variable
@@ -663,35 +663,35 @@ class TestXCOMOperations:
             pytest.param({"key": "test_key", "value": {"key2": "value2"}}, id="nested-dict-value"),
         ],
     )
-    @mock.patch("time.sleep", return_value=None)  # To have retries not slowing down tests
-    def test_xcom_get_success(self, mock_sleep, value):
-        # Simulate a successful response from the server when getting an xcom
-        # ...including a validation that retry really works
-        call_count = 0
+    def test_xcom_get_success(self, value):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            # Simulate a successful response from the server when getting an xcom
+            # ...including a validation that retry really works
+            call_count = 0
 
-        def handle_request(request: httpx.Request) -> httpx.Response:
-            nonlocal call_count
-            call_count += 1
-            if call_count < 3:
-                return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
-            if request.url.path == "/xcoms/dag_id/run_id/task_id/key":
-                return httpx.Response(
-                    status_code=201,
-                    json={"key": "test_key", "value": value},
-                )
-            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+            def handle_request(request: httpx.Request) -> httpx.Response:
+                nonlocal call_count
+                call_count += 1
+                if call_count < 3:
+                    return httpx.Response(status_code=500, json={"detail": "Internal Server Error"})
+                if request.url.path == "/xcoms/dag_id/run_id/task_id/key":
+                    return httpx.Response(
+                        status_code=201,
+                        json={"key": "test_key", "value": value},
+                    )
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
 
-        client = make_client(transport=httpx.MockTransport(handle_request))
-        result = client.xcoms.get(
-            dag_id="dag_id",
-            run_id="run_id",
-            task_id="task_id",
-            key="key",
-        )
-        assert isinstance(result, XComResponse)
-        assert result.key == "test_key"
-        assert result.value == value
-        assert call_count == 3
+            client = make_client(transport=httpx.MockTransport(handle_request))
+            result = client.xcoms.get(
+                dag_id="dag_id",
+                run_id="run_id",
+                task_id="task_id",
+                key="key",
+            )
+            assert isinstance(result, XComResponse)
+            assert result.key == "test_key"
+            assert result.value == value
+            assert call_count == 3
 
     def test_xcom_get_success_with_map_index(self):
         # Simulate a successful response from the server when getting an xcom with map_index passed
@@ -742,29 +742,29 @@ class TestXCOMOperations:
         assert result.key == "test_key"
         assert result.value == "test_value"
 
-    @mock.patch("time.sleep", return_value=None)
-    def test_xcom_get_500_error(self, mock_sleep):
-        # Simulate a successful response from the server returning a 500 error
-        def handle_request(request: httpx.Request) -> httpx.Response:
-            if request.url.path == "/xcoms/dag_id/run_id/task_id/key":
-                return httpx.Response(
-                    status_code=500,
-                    headers=[("content-Type", "application/json")],
-                    json={
-                        "reason": "invalid_format",
-                        "message": "XCom value is not a valid JSON",
-                    },
-                )
-            return httpx.Response(status_code=400, json={"detail": "Bad Request"})
+    def test_xcom_get_500_error(self):
+        with time_machine.travel("2023-01-01T00:00:00Z", tick=False):
+            # Simulate a successful response from the server returning a 500 error
+            def handle_request(request: httpx.Request) -> httpx.Response:
+                if request.url.path == "/xcoms/dag_id/run_id/task_id/key":
+                    return httpx.Response(
+                        status_code=500,
+                        headers=[("content-Type", "application/json")],
+                        json={
+                            "reason": "invalid_format",
+                            "message": "XCom value is not a valid JSON",
+                        },
+                    )
+                return httpx.Response(status_code=400, json={"detail": "Bad Request"})
 
-        client = make_client(transport=httpx.MockTransport(handle_request))
-        with pytest.raises(ServerResponseError):
-            client.xcoms.get(
-                dag_id="dag_id",
-                run_id="run_id",
-                task_id="task_id",
-                key="key",
-            )
+            client = make_client(transport=httpx.MockTransport(handle_request))
+            with pytest.raises(ServerResponseError):
+                client.xcoms.get(
+                    dag_id="dag_id",
+                    run_id="run_id",
+                    task_id="task_id",
+                    key="key",
+                )
 
     @pytest.mark.parametrize(
         "values",
@@ -1282,7 +1282,7 @@ class TestHITLOperations:
             params=None,
             multiple=False,
         )
-        assert isinstance(result, HITLDetailRequestResult)
+        assert isinstance(result, HITLDetailRequest)
         assert result.ti_id == ti_id
         assert result.options == ["Approval", "Reject"]
         assert result.subject == "This is subject"
@@ -1349,3 +1349,36 @@ class TestHITLOperations:
         assert result.params_input == {}
         assert result.responded_by_user == HITLUser(id="admin", name="admin")
         assert result.responded_at == timezone.datetime(2025, 7, 3, 0, 0, 0)
+
+
+class TestSSLContextCaching:
+    @pytest.fixture(autouse=True)
+    def clear_ssl_context_cache(self):
+        Client._get_ssl_context_cached.cache_clear()
+        yield
+        Client._get_ssl_context_cached.cache_clear()
+
+    def test_cache_hit_on_same_parameters(self):
+        ca_file = certifi.where()
+        ctx1 = Client._get_ssl_context_cached(ca_file, None)
+        ctx2 = Client._get_ssl_context_cached(ca_file, None)
+        assert ctx1 is ctx2
+
+    def test_cache_miss_if_cache_cleared(self):
+        ca_file = certifi.where()
+        ctx1 = Client._get_ssl_context_cached(ca_file, None)
+        Client._get_ssl_context_cached.cache_clear()
+        ctx2 = Client._get_ssl_context_cached(ca_file, None)
+        assert ctx1 is not ctx2
+
+    def test_cache_miss_on_different_parameters(self):
+        ca_file = certifi.where()
+
+        ctx1 = Client._get_ssl_context_cached(ca_file, None)
+        ctx2 = Client._get_ssl_context_cached(ca_file, ca_file)
+
+        info = Client._get_ssl_context_cached.cache_info()
+
+        assert ctx1 is not ctx2
+        assert info.misses == 2
+        assert info.currsize == 2
